#ifndef FAMILY_H
#define FAMILY_H

#include <stdio.h>
#include <gsl/gsl_rng.h>

#define CHECK_EF(fam, var) assert((void*)var->ef == (void*)fam)
#define CHECK_TYPE(var, t) assert(var->type == t)

#define FAMILY\
  char*     name;\
  unsigned int dim;\
  double  (*pdf           )(family* ef, double* x, param* param);\
  double* (*iter_pdf      )(family* ef, unsigned int n, double* in, param* lambda, double* out);\
  void    (*lambda2theta  )(family* ef, param* lambda, param* theta );\
  void    (*theta2lambda  )(family* ef, param* theta , param* lambda);\
  void    (*lambda2eta    )(family* ef, param* lambda, param* eta   );\
  void    (*eta2lambda    )(family* ef, param* eta   , param* lambda);\
  void    (*theta2eta     )(family* ef, param* theta , param* eta);\
  void    (*eta2theta     )(family* ef, param* eta   , param* theta);\
  double  (*F             )(family* ef, param* theta);\
  void    (*gradF         )(family* ef, param* theta, param* eta);\
  double  (*G             )(family* ef, param* theta);\
  void    (*gradG         )(family* ef, param* theta, param* eta);\
  param*  (*mle           )(family* ef, unsigned int n, double* data, char* mask, param* eta);\
  void*   (*new_param     )(family* ef, enum type type);\
  void    (*delete_param  )(family* ef, param* p);\
  void    (*copy_param    )(family* ef, param* src, param* dest);\
  void    (*t             )(family* ef, double* x, param* eta);\
  param*  (*add           )(family* ef, param* p, param* q, param* dest);\
  double  (*scalar        )(family* ef, param* p, param* q);\
  param*  (*minus         )(family* ef, param* p, param* q, param* dest);\
  param*  (*scale         )(family* ef, param* p, double a, param* dest);\
  param*  (*zero          )(family* ef, param* p);\
  double* (*rand          )(family* ef, param* p, gsl_rng* rng);\
  param*  (*as_source     )(family* ef, param* in);\
  param*  (*as_natural    )(family* ef, param* in);\
  param*  (*as_expectation)(family* ef, param* in);\
  void    (*fprint        )(FILE* stream, family* ef, param* param);\

typedef struct family family;
struct family {
  FAMILY
};

#endif

